#-----------------------------------------------------------------------------
# Strip zero morphs from FC2 or PZ2
# Aug 14, 2005
# For Shadownet
# Copyright (c) 2005 David G. Drumright (ockham)
#-----------------------------------------------------------------------------

from Tkinter import *
import tkFileDialog
import os
import string

scene=None
try:
   import poser
   scene = poser.Scene()
except:
   scene=None


#-----------------------------------------------------------------------------
if scene:
    BasePathUncut = poser.AppLocation()
    BasePath = BasePathUncut[:(-len("poser.exe"))]
    FacePath=os.path.normpath(BasePath+'/runtime/libraries/face/*.FC2')
    PosePath=os.path.normpath(BasePath+'/runtime/libraries/pose/*.PZ2')
else:
    FacePath = './*.FC2'
    PosePath = './*.PZ2'

#-----------------------------------------------------------------------------
# Next section is TK
#------------------------------------------------------------

class App:
    def __init__(self, master, textMessage):
        self.master = master
        master.title("Strip zero morphs")

        self.StatusEntry = Entry(self.master,width=80)
        self.StatusEntry.grid(row=0,column=0,columnspan=3)

        # Go/nogo buttons.
        self.buttonGo1 = Button(self.master, text="Find PZ2", command=self.HandleGoPZ2)
        self.buttonGo1.grid(row=3, column=0, pady=2,padx=2)

        self.buttonGo2 = Button(self.master, text="Find FC2", command=self.HandleGoFC2)
        self.buttonGo2.grid(row=3, column=1, pady=2,padx=2)

        self.buttonCancel = Button(self.master, text="Cancel", command=self.HandleCancel)
        self.buttonCancel.grid(row=3, column=2, pady=2,padx=2)

        self.master.protocol('WM_DELETE_WINDOW', self.HandleCancel)


    # - - - - - - - - - - - - - - - - - -

    def ShowStatus(self,S): # Just saving repetition
        self.StatusEntry.delete(0,END)
        self.StatusEntry.insert(0,S)
        self.StatusEntry.update_idletasks()

    # - - - - - - - - - - - - - - - - - -

    def Update(self):
        if scene: scene.ProcessSomeEvents()
        root.lift()
        root.after(100, self.Update)

    # - - - - - - - - - - - - - - - - - -

    def FileAction(self,fn):
        try:
            Front,Ext=os.path.splitext(fn)
        except:
            self.ShowStatus("Problem with file %s" % fn)
            return
        OutName=Front+'_mod'+Ext  # Poo.FC2 -> Poo_mod.FC2

        self.ShowStatus('working ...')
        fpIn=open(fn,'rt')
        fpOut=open(OutName,'wt')
        # Find morphs with zero settings, eliminate, rewrite
        InTarget=0
        BuildString=''
        BracketCount=0
        NonZero=1
        while 1:
            st=fpIn.readline()
            if not st: break
            Parts=string.split(string.strip(st))
            if Parts and (Parts[0]=='targetGeom'):
                InTarget=1
                BracketCount=0
                NonZero=1 # Assume good for now
            if InTarget:
                # Now we count brackets, watching for k 0 0.
                # If we have k 0 0, leave out the whole thing.
                # If k 0 nonzero, write as is.
                BuildString=BuildString+st # Add each line when InTarget
                Parts=string.split(string.strip(st))
                if Parts and (Parts[0]=='k'):
                    if float(Parts[2]) == 0.0:
                        NonZero=0
                elif string.count(st,'{'):
                    BracketCount=BracketCount+1
                elif string.count(st,'}'):
                    BracketCount=BracketCount-1
                    # Note that the If below must happen during the check for last bracket,
                    # not outside; when we check at the more general level, we end up turning
                    # off InTarget prematurely.
                    if (BracketCount==0): # Done with brackets.  Skip or write
                        if NonZero: # Done with brackets, nonzero, so write.
                           fpOut.write(BuildString)
                           BuildString=''
                           InTarget=0
                        else: # Just reset stuff without writing
                           BuildString=''
                           NonZero=1
                           InTarget=0

            else: # not InTarget
                fpOut.write(st)

        fpIn.close()
        fpOut.close()
        self.ShowStatus('Done with %s' % OutName)

    # - - - - - - - - - - - - - - - - - -

    def HandleNameFC2(self):
        # Pop up file box to get name.
        fn=tkFileDialog.askopenfilename(initialfile=FacePath, defaultextension=[("FC2 file", "*.FC2 *.fc2")] )

        if not fn:
            self.ShowStatus("No file selected. Try again.")
            return 0
        else: return os.path.abspath(fn)

    # - - - - - - - - - - - - - - - - - -

    def HandleNamePZ2(self):
        # Pop up file box to get name.
        fn=tkFileDialog.askopenfilename(initialfile=PosePath, defaultextension=[("PZ2 file", "*.PZ2 *.pz2")] )

        if not fn:
            self.ShowStatus("No file selected. Try again.")
            return 0

        else: return os.path.abspath(fn)

    # - - - - - - - - - - - - - - - - - -

    def HandleGoPZ2(self):
        # Action here
        fn=self.HandleNamePZ2()
        self.FileAction(fn)

    # - - - - - - - - - - - - - - - - - -

    def HandleGoFC2(self):
        # Action here
        fn=self.HandleNameFC2()
        self.FileAction(fn)

    # - - - - - - - - - - - - - - - - - -

    def HandleCancel(self):
        self.master.destroy()

#------------------------------------------------------------
# Activate the loop
#------------------------------------------------------------

root = Tk()
app = App(root, '')
app.Update()
root.mainloop()

#------------------------------------------------------------
# End TK loop.
#-----------------------------------------------------------------------------
